///////////////////////////////////////////////////////////////////////////////////////////////////
//
// Menu Class Implementation
//
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
//
// Copyright 1987-2022 PARMAVEX SERVICES
//
// Licensed under the European Union Public Licence (EUPL), Version 1.1 or -
// as soon they will be approved by the European Commission - subsequent
// versions of the EUPL (the "Licence"). You may not use this work except in
// compliance with the Licence. You may obtain a copy of the Licence at:
//
// http://ec.europa.eu/idabc/eupl
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the Licence is distributed on an "AS IS" basis,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the Licence for the specific language governing permissions and
// limitations under the Licence. This source code is free software. It
// must not be sold, leased, rented, sub-licensed or used for any form of
// monetary recompense whatsoever. This notice must not be removed or altered
// from this source distribution.
//
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Remarks
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Include Files
///////////////////////////////////////////////////////////////////////////////////////////////////

// 1. Own Interface

// 2. C System Files

// 3. C++ System Files

// 4. Other Libraries

// 5. This Project
#include "PxsBase/Header Files/Menu.h"

///////////////////////////////////////////////////////////////////////////////////////////////////
// Construction/Destruction
///////////////////////////////////////////////////////////////////////////////////////////////////

// Default constructor
Menu::Menu()
     :m_hMenu( nullptr ),
      m_menuItemInfo()    // Non-op
{
    // Init the structure
    memset( &m_menuItemInfo, 0, sizeof ( m_menuItemInfo ) );
    m_menuItemInfo.cbSize = sizeof ( m_menuItemInfo );
}

// Copy constructor
Menu::Menu( const Menu& oMenu )
     :Menu()
{
    *this = oMenu;
}

// Destructor
Menu::~Menu()
{
    // Clean up
    if ( m_hMenu )
    {
        DestroyMenu( m_hMenu );
    }
}

///////////////////////////////////////////////////////////////////////////////////////////////////
// Operators
///////////////////////////////////////////////////////////////////////////////////////////////////

// Assignment operator
Menu& Menu::operator= ( const Menu& oMenu )
{
    if ( this == &oMenu) return *this;

    m_hMenu = oMenu.m_hMenu;
    memcpy( &m_menuItemInfo, &oMenu.m_menuItemInfo, sizeof ( m_menuItemInfo ) );

    return *this;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
// Public Methods
///////////////////////////////////////////////////////////////////////////////////////////////////

//===============================================================================================//
//  Description:
//      Get the handle of the menu or menu item
//
//  Parameters:
//      None
//
//  Returns:
//      HMENU
//===============================================================================================//
HMENU Menu::GetMenuHandle() const
{
    return m_hMenu;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
// Protected Methods
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Private Methods
///////////////////////////////////////////////////////////////////////////////////////////////////
